// Copyright (c) 2021 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include <dawn/dawn.h>
#include "gametime.h"
#include "../display/render.h"
#include "../display/camera.h"
#include "../display/shader.h"
#include "../display/gui/font.h"
#include "../file/asset.h"
#include "../input/input.h"
#include "../debug/log.h"
#include "../poker/action/action.h"
#include "../poker/action/start.h"
#include "../poker/render/player.h"
#include "../poker/render/card.h"
#include "../poker/render/chip.h"
#include "../poker/render/frame.h"
#include "../poker/render/look.h"
#include "../poker/render/world.h"

/**
 * Initialize the game context.
 * 
 * @return True if successful, otherwise false.
 */
bool gameInit();

/**
 * Tick the main game loop.
 * 
 * @param platformDelta The platform tick delta between the last render.
 * @return True if successful, false if safe exit requested..
 */
bool gameUpdate(float platformDelta);

/**
 * Cleanup the game instance.
 */
void gameDispose();